@@ -115,19 +115,19 @@ Math.ArrowDatum <- function(x, ..., base = exp(1), digits = 0) {
   switch(.Generic,
     abs = eval_array_expression("abs_checked", x),
     ceiling = eval_array_expression("ceil", x),
-    sign = ,
-    floor = ,
-    trunc = ,
-    acos = ,
-    asin = ,
-    atan = ,
-    cos = ,
-    sin = ,
-    tan = {
-      eval_array_expression(.Generic, x)
-    },
+    sign = eval_array_expression("sign", x),
+    floor = eval_array_expression("floor", x),
+    trunc = eval_array_expression("trunc", x),
+    acos = eval_array_expression("acos_checked", x),
+    asin = eval_array_expression("asin_checked", x),
+    atan = eval_array_expression("atan", x),
+    cos = eval_array_expression("cos_checked", x),
+    sin = eval_array_expression("sin_checked", x),
+    tan = eval_array_expression("tan_checked", x),
     log = eval_array_expression("logb_checked", x, base),
     log10 = eval_array_expression("log10_checked", x),
+    log2 = eval_array_expression("log2_checked", x),
+    log1p = eval_array_expression("log1p_checked", x),
     round = eval_array_expression(
       "round",
       x,
@@ -135,9 +135,12 @@ Math.ArrowDatum <- function(x, ..., base = exp(1), digits = 0) {
     ),
     sqrt = eval_array_expression("sqrt_checked", x),
     exp = eval_array_expression("power_checked", exp(1), x),
+    cumsum = eval_array_expression("cumulative_sum_checked", x),
+    cumprod = eval_array_expression("cumulative_prod_checked", x),
+    cummax = eval_array_expression("cumulative_max", x),
+    cummin = eval_array_expression("cumulative_min", x),
     signif = ,
     expm1 = ,
-    log1p = ,
     cospi = ,
     sinpi = ,
     tanpi = ,
@@ -151,10 +154,6 @@ Math.ArrowDatum <- function(x, ..., base = exp(1), digits = 0) {
     gamma = ,
     digamma = ,
     trigamma = ,
-    cumsum = ,
-    cumprod = ,
-    cummax = ,
-    cummin = ,
     stop(paste0("Unsupported operation on `", class(x)[1L], "` : "), .Generic, call. = FALSE)
   )
 }
