@@ -189,10 +189,10 @@ test_that("ChunkedArray handles NaN", {
 
 test_that("ChunkedArray supports logical vectors (ARROW-3341)", {
   # with NA
-  data <- purrr::rerun(3, sample(c(TRUE, FALSE, NA), 100, replace = TRUE))
+  data <- purrr::map(1:3, ~ sample(c(TRUE, FALSE, NA), 100, replace = TRUE))
   expect_chunked_roundtrip(data, bool())
   # without NA
-  data <- purrr::rerun(3, sample(c(TRUE, FALSE), 100, replace = TRUE))
+  data <- purrr::map(1:3, ~ sample(c(TRUE, FALSE), 100, replace = TRUE))
   expect_chunked_roundtrip(data, bool())
 })
 
@@ -475,11 +475,6 @@ test_that("Handling string data with embedded nuls", {
   )
   chunked_array_with_nul <- ChunkedArray$create(raws)$cast(utf8())
 
-  # The behavior of the warnings/errors is slightly different with and without
-  # altrep. Without it (i.e. 3.5.0 and below, the error would trigger immediately
-  # on `as.vector()` where as with it, the error only happens on materialization)
-  skip_on_r_older_than("3.6")
-
   v <- expect_error(as.vector(chunked_array_with_nul), NA)
 
   expect_error(
